(*

  TRegValues v2.2
  (c) 1999 Ray Adams
  E-Mail: rayadams@writeme.com
  WEB   : http://rayadams.i.am


*)
unit RegValues;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry, DsgnIntf;

type
  ENameInvalid = class(Exception);
    TRootKey=(
              rkHKEY_CURRENT_USER,
              rkHKEY_CLASSES_ROOT,
              rkHKEY_LOCAL_MACHINE,
              rkHKEY_USERS,
              rkHKEY_PERFORMANCE_DATA,
              rkHKEY_CURRENT_CONFIG,
              rkHKEY_DYN_DATA
                             );
     TValueType=(vtFloat,vtBoolean,vtString,vtInteger,vtDateTime
                 );
{ Forward declarations }
type

  TOnGetDefValueEvent = procedure(Sender: TObject; var DefValue: variant) of object;

TRegValues = class;
{ Declarations }
TRegValue = class(TCollectionItem)
  private
  FOnErrorOpenKey:TNotifyEvent;
  FOnGetDefValue:TOnGetDefValueEvent;
  FOwnerReg:TRegValues;
  FSubKey:String;
  FValueName:String;
  FValueType:TValueType;
  FValue:Variant;
  FDefValue:Variant;
  FAutoStore:boolean;
  procedure _Error;virtual;
  protected
    { Protected declarations }
    function GetAsBoolean: Boolean; virtual;
    function GetAsDateTime: TDateTime; virtual;
    function GetAsFloat: Double; virtual;
    function GetAsInteger: Longint; virtual;
    function GetAsString: string; virtual;
    procedure SetAsBoolean(Value: Boolean); virtual;
    procedure SetAsDateTime(Value: TDateTime); virtual;
    procedure SetAsFloat(Value: Double); virtual;
    procedure SetAsInteger(Value: Longint); virtual;
    procedure SetAsString(const Value: string); virtual;
    procedure SetValue(sValue:String);
    function GetValue:string;
    procedure SetDefValue(sValue:String);
    function GetDefValue:string;
    function GetDisplayName: string; override;
    procedure SetValueType(vType:TValueType);virtual;
  public
    { Public declarations }
    constructor Create(Collection: TCollection);override;
    destructor Destroy; override;
    procedure SaveValue;virtual;
    procedure LoadValue;virtual;
    procedure SetValueName(sName:String);virtual;
    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsFloat: Double read GetAsFloat write SetAsFloat;
    property AsInteger: Longint read GetAsInteger write SetAsInteger;
    property AsString: string read GetAsString write SetAsString;
    property Value:Variant read FValue write FValue;
  published
    { Published declarations }
  property OnErrorOpenKey:TNotifyEvent read FOnErrorOpenKey write FOnErrorOpenKey;
  property OnGetDefValueEvent:TOnGetDefValueEvent read FOnGetDefValue write FOnGetDefValue;
  property SubKey:String read FSubKey write FSubKey;
  property ValueType:TValueType read FValueType write SetValueType;
  property DefValue:String read GetDefValue write SetDefValue;
  property ValueName:String read FValueName write SetValueName;
  property AutoStore:boolean read   FAutoStore write FAutoStore;
  end;

{TRegCollection}
TRegCollection=class(TCollection)
   private
     FOwner: TPersistent;
   protected
     function GetOwner: TPersistent; override;
   public
     procedure Assign(Source:TRegCollection);
     function GetItem(Index:Integer):TRegValue;
     property Items[Index: Integer]: TRegValue read GetItem; default;
     constructor Create(AOwner: TPersistent);
     function GetByName(sName:String):TRegValue;virtual;
end;
{TRegValues}

  TRegValues = class(TComponent)
  private
    { Private declarations }
    FRootKey:TRootKey;
    FKey:String;
    FCanCreate:boolean;
    FAutoLoad:boolean;
    FAutoSave:boolean;
    FR:TRegistry;
    FValues:TRegCollection;
    procedure SetRootKey(rkKey:TRootKey);
    function GetRootKey:TRootKey;virtual;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;
    procedure LoadValues;virtual;
    procedure SaveValues;virtual;
    function Get(sName:String):variant;
    procedure SetV(sName:string;Value:variant);
    procedure Clear;
    property Item[sName:String]:variant read Get write SetV;default;
    procedure Assign(Source: TRegValues); 
  published
    { Published declarations }
    property RootKey:TRootKey read GetRootKey write SetRootKey;
    property Key:String read FKey write FKey;
    property CanCreate:boolean read FCanCreate write FCanCreate;
    property AutoLoad:boolean read FAutoLoad write FAutoLoad;
    property AutoSave:boolean read FAutoSave write FAutoSave;
    property Items: TRegCollection read FValues write FValues;

  end;

{Component editor}
TRegValueEditor = class(TDefaultEditor)
protected
  procedure EditProperty(PropertyEditor: TPropertyEditor;
  var Continue, FreeEditor: Boolean);override;
public
  procedure ExecuteVerb(Index: Integer); override;
  function GetVerb(Index: Integer): string; override;
  function GetVerbCount: Integer; override;
end;

{Property editor}
TRegEditor = class(TComponentProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue:string; override;

           end;

//-------------------------------           
procedure Register;


implementation
{TRegCollection}

function TRegCollection.GetByName;
var I:Integer;
begin
for i:=0 to count-1 do
if Ansiuppercase(items[i].ValueName)=Ansiuppercase(sName) then
   begin
   result:=items[i];
   exit;
   end;
raise ENameInvalid.Create('Invalid Value Name!'+sName);
end;


function TRegCollection.GetItem;
begin
result:=inherited Items[Index] as TRegValue;
end;

constructor TRegCollection.Create;
begin
inherited Create(TRegValue);
FOwner:=AOwner;
end;


function TRegCollection.GetOwner;
begin
result:=FOwner;
end;
(******************************)
constructor TRegValues.Create;
begin
inherited Create(AOwner);
Fr:=TRegistry.Create;
FValues:=TRegCollection.Create(Self);
Key:='SOFTWARE\Your Company\Your Program';
FAutoSave:=true;
FAutoLoad:=true;
FCanCreate:=true;
if not (csDesigning in ComponentState) then
   begin
   LoadValues;
   end;

end;

destructor TRegValues.Destroy;
begin
if not (csDesigning in ComponentState) then
   begin
   SaveValues;
   end;
FValues.Free;
Fr.Free;
inherited Destroy;
end;

procedure TRegValues.LoadValues;
var I:Longint;
begin
for i:=0 to FValues.Count-1 do
    begin
        RootKey:=RootKey;
        FValues.items[i].LoadValue;
    end;

end;

procedure TRegValues.SaveValues;
var I:Longint;
begin
for i:=0 to FValues.Count-1 do
    begin
     if FValues.items[i].AutoStore then
     begin
     RootKey:=RootKey;
     FValues.items[i].SaveValue;
     end;
    end;
end;

procedure TRegValues.SetRootKey;
begin
FRootKey:=rkKey;
     case rkKey of
     rkHKEY_CLASSES_ROOT:FR.RootKey:=HKEY_CLASSES_ROOT;
     rkHKEY_CURRENT_USER:FR.RootKey:=HKEY_CURRENT_USER;
     rkHKEY_LOCAL_MACHINE:FR.RootKey:=HKEY_LOCAL_MACHINE;
     rkHKEY_USERS:FR.RootKey:=HKEY_USERS;
     rkHKEY_PERFORMANCE_DATA:FR.RootKey:=HKEY_PERFORMANCE_DATA;
     rkHKEY_CURRENT_CONFIG:FR.RootKey:=HKEY_CURRENT_CONFIG;
     rkHKEY_DYN_DATA:FR.RootKey:=HKEY_DYN_DATA;
     end;
end;

function TRegValues.GetRootKey;
begin
     case FR.RootKey of
     HKEY_CLASSES_ROOT:result:=rkHKEY_CLASSES_ROOT;
     HKEY_CURRENT_USER:result:=rkHKEY_CURRENT_USER;
     HKEY_LOCAL_MACHINE:result:=rkHKEY_LOCAL_MACHINE;
     HKEY_USERS:result:=rkHKEY_USERS;
     HKEY_PERFORMANCE_DATA:result:=rkHKEY_PERFORMANCE_DATA;
     HKEY_CURRENT_CONFIG:result:=rkHKEY_CURRENT_CONFIG;
     HKEY_DYN_DATA:result:=rkHKEY_DYN_DATA;
      else result:=rkHKEY_CURRENT_USER;
     end;
end;

function TRegValues.Get;
begin
Result:=FValues.getByName(sName).Value;
end;

procedure TRegValues.SetV;
begin
FValues.getByName(sName).Value:=Value;
end;

procedure TRegValues.Clear;
begin
FR.CloseKey;
//FR.OpenKey(FKey,false);
if FR.DeleteKey(FKey) then
                      messagebox(0,'All data removed from registry','Result',MB_OK)
                      else
                      messagebox(0,'Error removing data!','Result',MB_OK or MB_ICONERROR);                      
end;

{************************** TRegValue *******************}

constructor TRegValue.Create;
begin
inherited Create(Collection);
FOWnerReg:=TRegValues(TRegCollection(Collection).FOWner);
FValueType:=vtString;
FAutoStore:=true;
end;

destructor TRegValue.Destroy;
begin
inherited Destroy;
end;

procedure TRegValue._Error;
begin
if Assigned(FOnErrorOpenKey) then FOnErrorOpenKey(Self);
end;

procedure TRegValue.SaveValue;
var bResult:boolean;
    oldValue:variant;
begin
oldValue:=FValue;
if (csDesigning in FOWnerReg.ComponentState) then
   begin
   FValue:=FDefValue;
   end;
  {Save value to registry}
  FOwnerReg.FR.CloseKey;
  if Length(FSubKey)<>0 then
    bResult:=FOwnerReg.FR.OpenKey(FOwnerReg.FKey+'\'+FSubKey,FOwnerReg.FCanCreate)
  else
    bResult:=FOwnerReg.FR.OpenKey(FOwnerReg.FKey,FOwnerReg.FCanCreate);
if bResult then
   begin
   with FOwnerReg do
     case FValueType of
     vtFloat:fr.WriteFloat(FValueName,FValue);
     vtBoolean:fr.WriteBool(FValueName,FValue);
     vtString:fr.WriteString(FValueName,FValue);
     vtInteger:fr.WriteInteger(FValueName,FValue);
     vtDateTime:fr.WriteDateTime(FValueName,FValue);
     end;
   end else _Error;

   if (csDesigning in FOWnerReg.ComponentState) then
   begin
   FValue:=oldValue;
   end;

end;

procedure TRegValue.LoadValue;
var bResult:boolean;
begin
  {Save value to registry}
FOwnerReg.FR.CloseKey;
  if Length(FSubKey)<>0 then
    bResult:=FOwnerReg.FR.OpenKey(FOwnerReg.FKey+'\'+FSubKey,FOwnerReg.FCanCreate)
  else
    bResult:=FOwnerReg.FR.OpenKey(FOwnerReg.FKey,FOwnerReg.FCanCreate);
if bResult then
   begin
   if FOwnerReg.FR.ValueExists(FValueName) then
   with FOwnerReg do
     case FValueType of
     vtFloat:FValue:=Double(fr.ReadFloat(FValueName));
     vtBoolean:FValue:=Boolean(fr.ReadBool(FValueName));
     vtString:FValue:=String(fr.ReadString(FValueName));
     vtInteger:FValue:=INteger(fr.ReadInteger(FValueName));
     vtDateTime:FValue:=TDateTime(fr.ReadDateTime(FValueName));
     end else
     //Getting default value if not assigned event
     if Assigned(FOnGetDefValue) then FOnGetDefValue(Self,FValue) else FValue:=FDefValue;
   end else _Error;
end;

procedure TRegValue.SetValue;
begin
  case FValueType of
  vtFloat:FValue:=StrToFloat(sValue);
  vtBoolean:if AnsiUpperCase(sValue)='TRUE' then FValue:=true else FValue:=false;
  vtString:FValue:=sValue;
  vtInteger:FValue:=StrToInt(sValue);
  vtDateTime:FValue:=StrToDateTime(sValue);
  end;
end;

function TRegValue.GetValue;
begin
if FValueType<>vtBoolean then
result:=string(FValue)
else if FValue then result:='TRUE' else result:='FALSE';
end;

procedure TRegValue.SetDefValue;
begin
  case FValueType of
  vtFloat:FDefValue:=StrToFloat(sValue);
  vtBoolean:if AnsiUpperCase(sValue)='TRUE' then FDefValue:=true else FDefValue:=false;
  vtString:FDefValue:=sValue;
  vtInteger:FDefValue:=StrToInt(sValue);
  vtDateTime:FDefValue:=StrToDateTime(sValue);
  end;
end;

function TRegValue.GetDefValue;
begin
if FValueType<>vtBoolean then
result:=string(FDefValue)
else if FDefValue then result:='TRUE' else result:='FALSE';
end;

procedure TRegValue.SetAsBoolean;
begin
if Value then SetValue('TRUE') else SetValue('FALSE');
end;

procedure TRegValue.SetAsDateTime;
begin
SetValue(DateTimeToStr(Value));
end;

procedure TRegValue.SetAsFloat;
begin
SetValue(FloatToStr(Value));
end;

procedure TRegValue.SetAsInteger;
begin
SetVAlue(IntToStr(Value));
end;

procedure TRegValue.SetAsString;
begin
SetValue(Value);
end;

function TRegValue.GetAsBoolean;
begin
result:=FValue<>0;
end;

function TRegValue.GetAsDateTime;
begin
result:=TDateTime(FValue);
end;

function TRegValue.GetAsFloat;
begin
result:=Double(FValue);
end;

function TRegValue.GetAsInteger;
begin
result:=Integer(FValue);
end;

function TRegValue.GetAsString;
begin
result:=string(FValue);
end;

function TRegValue.GetDisplayName;
begin
Result:=FValueName;
if Result='' then Result := inherited GetDisplayName;
end;

procedure TRegValue.SetValueName;
begin
DisplayName:=sName;
FValueName:=sName;
if (not (csDesigning in FOWnerReg.ComponentState)) and FOWnerReg.AutoLoad then
   begin
   LoadValue;
   end;
end;

procedure TRegValue.SetValueType;
begin
FValueType:=vType;
end;

procedure TRegCollection.Assign(Source: TRegCollection);
var I:Longint;
    rValue:TRegValue;
begin
{}
Clear;
for i:=1 to Source.Count do
    begin
    rValue:=Add as TRegValue;

           rValue.FOwnerReg:=GetOwner as TRegValues;
           rValue.FSubKey:=Source.Items[i-1].FSubKey;
           rValue.FValueName:=Source.Items[i-1].FValueName;
           rValue.FValueType:=Source.Items[i-1].FValueType;
           rValue.FValue:=Source.Items[i-1].FValue;
           rValue.FDefValue:=Source.Items[i-1].FDefValue;
           rValue.FAutoStore:=Source.Items[i-1].FAutoStore;
    end;
end;

{TRegValueEditor}

procedure TRegValueEditor.ExecuteVerb(Index: Integer);
begin
 case Index of
 0:Showmessage('(c) 1999 Ray Adams'^m'rayadams@writeme.com');
 2:begin
        Edit;
   end;
 3:begin
        (Component as TRegValues).SaveValues;
        messagebox(0,'All data stored in registry','Result',MB_OK)
        end;
 4:(Component as TRegValues).Clear;
 end;
end;

function TRegValueEditor.GetVerb(Index: Integer): string;
begin
  case Index of
  0:result:='About...';
  1:result:='-';
  2:result:='&Edit variables';
  3:result:='&Write to registry';
  4:result:='&Remove from registry';
  end;
end;

function TRegValueEditor.GetVerbCount;
begin
Result:=5;
end;

procedure TRegValueEditor.EditProperty(PropertyEditor: TPropertyEditor;
  var Continue, FreeEditor: Boolean);
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'Items') = 0) then
  begin
    PropertyEditor.Edit;
    Continue := False;
  end;
end;


{-----------------------------}

procedure Register;
begin
  RegisterComponents('Ray Adams', [TRegValues]);
  RegisterComponentEditor(TRegValues,TRegValueEditor);
//  RegisterPropertyEditor(TypeInfo(TRegCollection),TRegValues,'',TRegEditor);
end;

{ TRegEditor }

procedure TRegEditor.Edit;
begin

inherited edit;
end;

function TRegEditor.GetAttributes: TPropertyAttributes;
begin
Result := inherited GetAttributes + [paDialog];
end;

function TRegEditor.GetValue: string;
begin
Result:='[Values]';
end;

procedure TRegValues.Assign(Source: TRegValues);
begin
FRootKey:=(Source as TRegVAlues).FRootKey;
FKey:=(Source as TRegVAlues).FKey;
FCanCreate:=(Source as TRegVAlues).CanCreate;
FAutoLoad:=(Source as TRegVAlues).AutoLoad;
FAutoSave:=(Source as TRegVAlues).AutoSave;
//FR:=(Source as TRegVAlues).FR;
FValues.Assign(Source.FValues);

end;

end.

